'use strict';

var path = require("path");

var httpsTransMap = {
    // 原生支持HTTPS的服务-百度内
    "http://baike.baidu.com": "https://baike.baidu.com",
    "http://wapbaike.baidu.com": "https://wapbaike.baidu.com",
    "http://baike.bdimg.com": "https://bkssl.bdimg.com",
    "http://baikebcs.bdimg.com": "https://baikebcs.bdimg.com",
    "http://hiphotos.baidu.com": "https://imgsa.baidu.com",
    "http://imgsrc.baidu.com": "https://imgsa.baidu.com",
    "http://a.hiphotos.baidu.com": "https://imgsa.baidu.com",
    "http://b.hiphotos.baidu.com": "https://imgsa.baidu.com",
    "http://c.hiphotos.baidu.com": "https://imgsa.baidu.com",
    "http://d.hiphotos.baidu.com": "https://imgsa.baidu.com",
    "http://e.hiphotos.baidu.com": "https://imgsa.baidu.com",
    "http://f.hiphotos.baidu.com": "https://imgsa.baidu.com",
    "http://g.hiphotos.baidu.com": "https://imgsa.baidu.com",
    "http://h.hiphotos.baidu.com": "https://imgsa.baidu.com",
    "http://cdn00.baidu-img.cn": "https://timg01.bdimg.com",
    "http://m.baidu.com": "https://m.baidu.com",
    "http://drm.media.baidubce.com": "https://drm.media.baidubce.com",
    "http://bs.baidu.com": "https://baikebcs.bdimg.com",
    "http://passport.baidu.com": "https://passport.baidu.com",
    "http://wappass.baidu.com": "https://wappass.baidu.com",
    // 原生支持HTTPS的服务-百度外
    "http://jsapi.qq.com": "https://jsapi.qq.com",
    // 需要通过HTTPS代理的服务-百度内
    "http://img.baidu.com": "https://gss0.baidu.com/70cFsjip0QIZ8tyhnq",
    "http://himg.baidu.com" : "https://gss0.baidu.com/7Ls0a8Sm2Q5IlBGlnYG",
    "http://himg.bdimg.com" : "https://gss0.bdstatic.com/7Ls0a8Sm1A5BphGlnYG",
    "http://tts.baidu.com": "https://gss0.baidu.com/6KAZsjip0QIZ8tyhnq",
    "http://zhangmenshiting.baidu.com": "https://gss0.baidu.com/yrwHcj7d1QgTmNK7m9_UKT6zgBYnreHg-_",
    "http://nssug.baidu.com": "https://gsp0.baidu.com/8qUZeT8a2gU2pMbgoY3K",
    "http://msg.baidu.com": "https://gsp0.baidu.com/8_UFsjip0QIZ8tyhnq",
    "http://static.tieba.baidu.com": "https://gsp0.baidu.com/5aAHeD3nKhI2p27j8IqW0jdnxx1xbK",
    "http://imgdata.baidu.com": "https://gss2.bdstatic.com/70cFaDe52MgCo2Kml5_Y_D3",
    'http://bdimg.share.baidu.com': 'https://gss0.baidu.com/9rA4cT8aBw9FktbgoI7O1ygwehsv',
    'http://dispatcher.video.qiyi.com': 'https://gss0.bdstatic.com/-LsZfDe52w9JkxG9m9iS_HFjgAkrreHg-_',
    // 需要通过HTTPS代理的服务-百度外
    'http://widget.weibo.com': 'https://gss0.bdstatic.com/6bsCazu5Kh1Jm27h8IuM_a',
    'http://gcjgdrdrzymhp9e2xpz.exp.bcevod.com': 'https://gcjgdrdrzymhp9e2xpz.exp.bcevod.com'
}

// url的情况包含3种：1）纯http字符串, 2）字符串和smarty变量的结合，3）纯smarty变量
function urlReplace(url) {
    if (url.indexOf('http:') === 0) {
        if (/\{%\$.+%\}/.test(url)) {
            // 字符串和smarty变量的结合
            // 在s2那里加个括号是因为 {%'http://widget.weibo.com/weiboshow/index.php?language=&width=270&height=500&fansRow=2&followbtn=1&ptype=1&speed=0&skin=5&isTitle=1&noborder=1&isWeibo=1&isFans=0&uid='|cat:$weiboData['weiboId']|f_escape_path|cat:'&colors=fcfcfc,fcfcfc,666666,136ec2,ffffff&ch=spr_qdhz_bd_baidubaike_weibo_cj01&dpc=1'|wiki_common_https_trans%}
            // 上面的f_esacape_path 本来只会转义smarty 变量，但是经过cat 的拼接以后，之前的字符串也会被转义，会导致有些url或者iframe的连接无法访问
            // return url.replace(/(http:.*(?=\{%\$))\{%(.+)%\}(.*)/, function (word, s1, s2, s3) {
            //     return  '{%\'' + s1 + '\'|cat:(' + s2 + ')|cat:\'' + s3 + '\'|wiki_common_https_trans%}';
            // });

            // 只处理域名部分，上面那种方式有多个变量时不适用也不好改
            return url.replace(/(http:\/\/[^\/]+?)(\/.*)/, function (word, s1, s2) {
                return '{%\'' + s1 + '\'|wiki_common_https_trans%}' + s2;
            });
        } else {
            // 纯http字符串
            return '{%"' + url + '"|wiki_common_https_trans%}';
        }
    } else {
        // 纯smarty变量
        // 前面加^\{%\$是为了确保该url是一个smarty变量，而不是一个像{%if%}这种表达式
        return url.replace(/^\{%(\$.+)%\}/, '{%$1|wiki_common_https_trans%}');
    }
}

// 对tpl的处理
function httpsTplReplace(content) {
    // 替换src后的url
    return content.replace(/(?:src|img)="([^\s]+)"/gi, function (word, key) {
        var ret = word.replace(key, urlReplace(key));
        if (ret !== word) {
            console.log('🙆 ==> replacement SRC::' + '[' + word + ']' + ' => [' + ret + ']');
        }
        return ret;
    })

    // 替换json_encode的字符串
    .replace(/\{%.*?(\$(.+?)\|json_encode).*?%\}/gi, function(word, key){
        // eg. key = $btns|json_encode
        var ret = word.replace(key, key + '|wiki_common_https_trans');
        if( ret !== word) {
            console.log('💃 ==> replace JSON_ENCODE::' + '[' + word + ']' + ' => [' + ret + ']');
        }
        return ret;
    })

    // 替换模板中包含在background中的smarty的变量
    // 添加(?:[^;]*)以匹配这种字段： "background:{%$data.bgColor%} {%if $lang == 'en'%}url({%$data.en_bgImg|wiki_common_pic_src2url%}){%else%}url({%$data.bgImg|wiki_common_pic_src2url%}){%/if%} center center  no-repeat;"
    // 2016-09-05 yuhui
    .replace(/background(?:\-image)?:\s*(?:#(?:[A-Za-z0-9]{3}|[A-Za-z0-9]{6})\s+)?(?:[^;]*)url\(['"]?\{%(\$.*?)%\}['"]?\)/gi, function(word, key){
        var ret = word.replace(key, key + '|wiki_common_https_trans');
        if( ret !== word) {
            console.log('🏃 ==> replace BACKGROUND(1)::' + '[' + word + ']' + ' => [' + ret + ']');
        }
        return ret;
    })

    // 匹配 "background:{%$data.bgColor%} {%if $lang == 'en'%}url({%$data.en_bgImg|wiki_common_pic_src2url%}){%else%}url({%$data.bgImg|wiki_common_pic_src2url%}){%/if%} center center  no-repeat;"
    // 这种 url 和 background: 之间包涵其他字符，且后面可能包含多个url的字段
    .replace(/background(?:\-image)?:\s*([^;]*)/gi, function (word, key) {
        var newKey = key.replace(/url\(\{\%([^%]+)\%\}\)/gi, function (word, match1, index, key) {
            // 跟上一种有重合的地方
            if (!/\|wiki_common_https_trans/.test(match1)) {
                word = word.replace(match1, match1 + '|wiki_common_https_trans');
            }
            return word;
        });
        var ret = word.replace(key, newKey);
        if (ret !== word) {
            console.log('🏋 ==> replace BACNGROUND(2)::' + '[' + word + ']' + ' => [' + ret + ']');
        }
        return ret;
    })

    // 处理wap端lazy-img
    .replace(/class="lazy-img.*".*data-url="([^\s]+)"/gi, function (word, key) {
        var ret = word.replace(key, urlReplace(key));
        if (ret !== word) {
            console.log('🏄 ==> replace LAZY-IMG::' + '[' + word + ']' + ' => [' + ret + ']');
        }
        return ret;
    })

    // 处理 apple-touch-icon(-precomposed) 和 apple-touch-startup-image
    .replace(/rel="(?:apple-touch-icon|apple-touch-icon-precomposed|apple-touch-startup-image)".+href="([^\s]+)"/gi, function (word, key) {
        var ret = word.replace(key, urlReplace(key));
        if (ret !== word) {
            console.log('🏇 ==> replace APPLE-TOUCH-ICON::' + '[' + word + ']' + ' => [' + ret + ']')
        }
        return ret;
    })

    // 处理pc meta中 name="image" 和 itemprop="image" 后面 content="http://.."
    .replace(/<meta.+(?:name="image"|itemprop="image").+content="([^\s]+)"/gi, function (word, key) {
        var ret = word.replace(key, urlReplace(key));
        if (ret !== word) {
            console.log('🏌 ==> replace META::' + '[' + word + ']' + ' => [' + ret + ']');
        }
        return ret;
    });
};

function httpsCssReplace(content) {
    // 匹配出 http://d.hiphotos.baidu.com 这种url host，并替换为对应的https url host
    return content.replace(/background(?:\-image)?:\s*(?:#(?:[A-Za-z0-9]{3}|[A-Za-z0-9]{6})\s+)?url\((http:\/\/[\w\.\-]+).*?\)/gi, function(word, key){
        var ret = httpsTransMap[key] ? word.replace(key, httpsTransMap[key]) : word;
        if( ret !== word) {
            console.log('💖 ==> replace CSS::' + '[' + word + ']' + ' => [' + ret + ']');
        }
        return ret;
    });
}

function httpsJSReplace(content) {
    // var reg = /\'\s*(http:[\w\d\.\-#\?&=\/]+?)\s*?\'|\"\s*(http:[\w\d\.\-#\?&=\/]+?)\s*?\"/gi;
    // 增加%\{\}\$_\| 以匹配tpl在jsmart中被__inline进去的情况
    var reg = /\'\s*(http:[\w\d\.\-#\?&=\/%\{\}\$_\|]+)\s*?\'|\"\s*(http:[\w\d\.\-#\?&=\/%\{\}\$_\|]+)\s*?\"/gi;
    return content.replace(reg, function(word, match1, match2){
        var match = match1 || match2;
        var origin = word;
        var ret = match.replace(/^(http:\/\/[\w\d\.\-]+).*/, function(url, host){
            return httpsTransMap[host] ? url.replace(host, httpsTransMap[host]) : url;
        });
        if(ret !== match){
            word = word.replace(match, ret);
            console.log('💋 ==> replace JS::' + '[' + origin + ']' + ' => [' + word + ']');
        }
        return word;
    });
}


var exports = module.exports = function(ret, conf, settings, opt) {
    
    var ld = settings.left_delimiter || fis.config.get('settings.smarty.left_delimiter') || '{%';
    var rd = settings.right_delimiter || fis.config.get('settings.smarty.right_delimiter') || '%}';
    
    var ids = ret.ids || {};

    fis.util.map(ids, function (src, file) {
        if ( /^\.tpl|\.js|\.css$/.test(file.rExt)) {
            var origin = file.getContent();
            var content = origin;
            switch (file.rExt) {
                case '.tpl':
                content = httpsTplReplace(content);
                break;

                case '.js':
                if (!Array.isArray(settings.exclude) || settings.exclude.indexOf(file.id) === -1) {
                    content = httpsJSReplace(content);
                }
                break;

                case '.css':
                content = httpsCssReplace(content);
                break;
            }
            if(content !== origin) {
                console.log('✨✨✨[LOG] Replacements completed in [ ' + file.id + ' ]✨✨✨');
            }
            file.setContent(content);
        }
    });
};